

/*---------------------------- Module Includes ----------------------------*/

#include <ME218_C32.h>
#include "ES_Configure.h"
#include "ES_General.h"
#include "ES_Events.h"
#include "ES_PostList.h"
#include "ES_Timers.h"
#include "ES_Framework.h"
#include "EventCheckers.h"
#include <stdio.h>
#include "ES_ServiceHeaders.h"
#include "DefaultState.h"


/*----------------------------- Module Defines ----------------------------*/

#define QUEUE_SIZE 10

/*--------------------------- Module Prototypes ---------------------------*/

DefaultState processKeyboard(uint16_t);

/*---------------------------- Module Variables ---------------------------*/

static DefaultState CurrentState;
static uint8_t MyPriority;

static int timerInterval;
static int basket;

/*---------------------------- Module Functions ---------------------------*/

// Initialization and Post functions taken from Ed Carryer's examples
boolean InitDefaultSM ( uint8_t Priority ) 
{ 
  ES_Event ThisEvent; 
  MyPriority = Priority; 

  DDRM = 0x38;
  DDRT = 0x0;
  PTM = 0x0;
  
   
   timerInterval = 1;
   
   basket = 0;
   
  if (ADS12_Init("AOOOOOOO") == ADS12_OK) 
  {
   
  } else 
  {
    printf("ADSInit Failed.\r\n");
  }
  
  LightLED(1);
  LightLED(2);
  LightLED(3);
  
  CurrentState = InitDefault;
  puts("Initialized Default");
  ThisEvent.EventType = ES_INIT; 
  if (ES_PostToService( MyPriority, ThisEvent) == True)
  { 
      puts("Returning True");
      return True; 
  } else { 
      return False; 
  } 
} 

boolean PostDefaultSM( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);

}

// Morse Code State Machine
ES_Event RunDefaultSM( ES_Event ThisEvent ) 
{ 
	DefaultState nextState = CurrentState;
	DefaultState keyState = NO_CHANGE;
	ES_Event ReturnEvent;
	ES_Event currEvent;
	ReturnEvent.EventType = ES_NO_EVENT;
	

   if (ThisEvent.EventType == ES_TIMEOUT && ThisEvent.EventParam == 1)
   {
      printf("Timed out.  %d\r\n", timerInterval);
      OffLED(timerInterval++);
      currEvent.EventType = Failure;
      if(timerInterval > 3) 
      {
         timerInterval=1;      
         PostDefaultSM(currEvent);
         sendMessage(6);
      } else 
      {
         ES_Timer_SetTimer(1,15000);
         ES_Timer_StartTimer(1);
      }
   }
   //These events just set the basket variable.
   if (ThisEvent.EventType == Basket1)
   {
      basket = 1;
   }
   if (ThisEvent.EventType == Basket0)
   {
      basket = 0;
   }
   //When Batman finds that Robin failed through a message, this will reset everything and put it back in lockdown.
   if (ThisEvent.EventType == Failure)
	   {
	      OffAll();
	      timerInterval=1;
		   ES_Timer_StopTimer(1);
	      nextState = Lockdown;
	   }
   
	switch (CurrentState)
	{
	   
	   case InitDefault:
	      puts("In InitDefault");
	      if (ThisEvent.EventType == ES_INIT)
	      {
	         nextState = Lockdown;
	      }
	      
	   
	      break;
	      
	   case Lockdown:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }
		  if (ThisEvent.EventType == BATID_FOUND)
	      {
	         LightLED(1);
	         LightLED(2);
	         LightLED(3);
	         ES_Timer_SetTimer(1,15000);
            ES_Timer_StartTimer(1);
	         nextState = WaitingForCode;
		      sendMessage(1);
	         puts("BatID DETECTED.\n\r");
	      }
	      break;


	   case WaitingForCode:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == CodeSuccess)
		  {
			  puts("Changing state to WaitingForIR.\r\n");
			  nextState = WaitingForIR;
			  if(basket)
			  {
			   LightLED(4);
			  } else 
			  {
			   LightLED(5);
			  }
		  }
		  break;

	   case WaitingForIR:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == IRDetected && ThisEvent.EventParam == basket)
		  {
			//Correct basket
			  OffLED(4);
		     OffLED(5);
			  puts("Changing state to WaitingForBall.\r\n");
			  sendMessage(3);
			  nextState = WaitingForBall;
			 
		  } else if (ThisEvent.EventType == IRDetected && ThisEvent.EventParam != basket)
		  {
			//Wrong basket
		     sendMessage(6);
		     currEvent.EventType = Failure;
		     timerInterval=1;
		     ES_Timer_StopTimer(1);
		     PostDefaultSM(currEvent);
		  }
		  break;

	   case WaitingForBall:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == BallSuccess)
		  {
			  puts("Changing state to WaitingForAnalog1.\r\n");
			  nextState = WaitingForAnalog1;
		  }
		  break;

	   case WaitingForAnalog1:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }
		//Code is 3-1-4, first is 3
		  if (ThisEvent.EventType == AnalogCode && ThisEvent.EventParam == 3)
		  {
			  puts("Changing state to WaitingForAnalog2.\r\n");
			  nextState = WaitingForAnalog2;
		  }
		  break;
		
		case WaitingForAnalog2:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == AnalogCode && ThisEvent.EventParam == 1)
		  {
			  puts("Changing state to WaitingForAnalog3.\r\n");
			  nextState = WaitingForAnalog3;
		  }
		  break;
		  
		case WaitingForAnalog3:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == AnalogCode && ThisEvent.EventParam == 4)
		  {
		  //Puts the machine back into lockdown after a celebration sequence.
			  puts("Changing state to Victory!.\r\n");
			  ES_Timer_StopTimer(1);
			  LightLED(1);
			  LightLED(2);
			  LightLED(3);
			  sendMessage(4);   
			  timerInterval=1;
		     ES_Timer_StopTimer(1);
		     Celebration();
			  basket = 0;
			  nextState = Lockdown;
		  }
		  break;    
	}
	CurrentState = nextState;

	return ReturnEvent;
} 


DefaultState QueryDefaultSM ()
{
   return(CurrentState);
}

DefaultState processKeyboard(uint16_t input)
{
   ES_Event currEvent;
   char key = input;
   DefaultState newState = NO_CHANGE;
   if (key == 'a')
   {
	  //LightLED(1);
	  currEvent.EventType = CodeSuccess;
	  PostDefaultSM(currEvent);
	  
   }   
   if (key == 'b')
   {
     //LightLED(2);
     // newState = Lockdown;
	  //puts("In Lockdown now.\n\r");
	  
	  currEvent.EventType = BallSuccess;
	  PostDefaultSM(currEvent);
	  
   }
   if (key == 'c')
   {
      //Turn off LED
	  OffLED(1);
	  OffLED(2);
	  OffLED(3);
      newState = WaitingForCode;
	  puts("In WaitingForCode now.\n\r");
   }
   if (key == '1')
   {
      sendMessage(1);
      LightLED(3);
	  puts("Sending Message 1.\n\r");	  	  
   }
   if (key == '2')
   {
      sendMessage(2);
	  puts("Sending Message 2.\n\r");      
   }
   if (key == '3')
   {
      sendMessage(3);
	  puts("Sending Message 3.\n\r");
   }
   if (key == '4')
   {
      sendMessage(4);
	  puts("Sending Message 4.\n\r");      
   }
   if (key == '5')
   {
      sendMessage(5);
	  puts("Sending Message 5.\n\r");      
   }
   if (key == '6')
   {
      sendMessage(6);
	  puts("Sending Message 6.\n\r");      
   }
   if (key == '7')
   {
      sendMessage(7);
	  puts("Sending Message 7.\n\r");      
   }
   if (key == '8')
   {
      sendMessage(8);
	  puts("Buzzer on!.\n\r");
	  DDRT |= BIT6HI;
	  PTT |= BIT6HI;      
   }
   if (key == '9')
   {
      sendMessage(9);
	  puts("Buzzer off!.\n\r");
     DDRT |= BIT6HI;
	  PTT &= BIT6LO;
   }
   return newState;
}