
/*|||||||||||||||||||||||||||||||||BATMAN'S EVENT CHECKER||||||||||||||||||||||||||||||||||*/

/* ----------------------------------- Module Includes ----------------------------------- */

#include <ME218_C32.h>
#include "ES_Configure.h"
#include "ES_General.h"
#include "ES_Events.h"
#include "ES_PostList.h"
#include "ES_Timers.h"
#include "EventCheckers.h"
#include <stdio.h>
#include "ES_ServiceHeaders.h"

/*----------------------------- Module Defines ----------------------------*/

//These values should only be used with the M port on the C32.

#define INPUT_MASK 0x07 //0x00000111
#define MSG1 0x01 //0x00000001
#define MSG2 0x02 //0x00000010
#define MSG3 0x04 //0x00000100
#define MSG4 0x03 //0x00000011
#define MSG5 0x05 //0x00000101
#define MSG6 0x06 //0x00000110
#define MSG7 0x07 //0x00000111

//These values should only be used with the T port on the C32.
/* Bat ID on PTT Pin 0, Start button on PTT Pin 1, IR1 on PTT Pin 3, IR2 on PTT Pin 4*/
#define BatID		0x01 //0x00000001
#define IR1			0x04 //0x00000100
#define IR2			0x08 //0x00001000

//These values should only be used with the AD port on the C32.
#define Analog    0xC0 //0x10000000

/*--------------------------- Module Prototypes ---------------------------*/



/*---------------------------- Module Variables ---------------------------*/



/*---------------------------- Module Functions ---------------------------*/

/*||||||||||||||||||||||||||ROBIN TO BATMAN MESSAGES|||||||||||||||||||||||*/

// Checks for input messages from second C32. Bits in INPUT_MASK will be pulsed to send a message.
// checkMessage simply checks for this pulse.
boolean checkMessage (void)
{
	static unsigned char lastInputState;

	unsigned char currentInputState;
	boolean returnVal = FALSE;
	ES_Event currEvent;
	currEvent.EventParam = 0;

	currentInputState = PTM & INPUT_MASK;
	if (currentInputState != lastInputState)
	{
		switch (currentInputState)
		{
			//Message 1 is ___Code successfully inputted by Robin___
			case MSG1:
				puts("Received ___Code successfully inputted by Robin___! \r\n");

				currEvent.EventType = CodeSuccess;
				PostDefaultSM(currEvent);

				break;

			//Message 2 is ___Correct Ball Switch hit by Robin___
			case MSG2:
				puts("Received ___Correct Ball Switch hit by Robin___! \r\n");

				currEvent.EventType = BallSuccess;
				PostDefaultSM(currEvent);

				break;

         //Message 3 is ___Basket is 1___
			case MSG3:
				puts("Received __Basket is 1__! \r\n");

				currEvent.EventType = Basket1;
				PostDefaultSM(currEvent);

				break;

			//Message 4 is ___Basket is 0___
			case MSG4:
				puts("Received __Basket is 0__! \r\n");

				currEvent.EventType = Basket0;
				PostDefaultSM(currEvent);

				break;
				
			case MSG5:
				puts("Received __Message 5__! \r\n");

				//currEvent.EventType = Msg1Event;
				//Post_____SM(currEvent);

				break;	
		
			case MSG6:
			   puts("Received ___Robin Failed___! \r\n");

				currEvent.EventType = Failure;
				PostDefaultSM(currEvent);
			//	puts("Received __Message 6__! \r\n");

				//currEvent.EventType = Msg1Event;
				//Post_____SM(currEvent);

				break;	
			
			//Message 7 is ___Robin Failed___
			case MSG7:
		//		puts("Received ___Robin Failed___! \r\n");

	//			currEvent.EventType = Failure;
		//		PostDefaultSM(currEvent);

				break;
				
			default:
			   break;

		}
	}
	lastInputState = currentInputState;
	return returnVal;

}


/*********************************************************/
/***************  Batman: Button Checker *****************/
/*********************************************************/

/*-------  Start Button  -------*/


/*************************************************/
/************  Check morse events ****************/
/*************************************************/

boolean CheckMorseEvents ( void )
{
   ES_Event NewEvent;
   static char LastInputState;
   static boolean ReturnVal = False;
   static char CurrentInputState;
   
   // Get CurrentState of the input line.
   CurrentInputState = PTT & BatID;

   // If the state of the Morse input line has changed.
   if (CurrentInputState != LastInputState)
   {
   // If the current state of the input line is high.
     if (CurrentInputState != 0)
     {
     // PostEvent RisingEdge with parameter CurrentTime.
 //        puts("Found Rising Edge.");
         NewEvent.EventType = RisingEdge;
         NewEvent.EventParam =  ES_Timer_GetTime();
         PostMorseElementsFSM(NewEvent);
         PostDecodeMorseService(NewEvent);
     }  
     
     // Else (current input state is low).
     else
     {
     // PostEvent FallingEdge with parameter CurrentTime.
         NewEvent.EventType = FallingEdge;
         NewEvent.EventParam =  ES_Timer_GetTime();
         PostMorseElementsFSM(NewEvent);
         PostDecodeMorseService(NewEvent);      
     }
     // Set ReturnVal = True.
     ReturnVal = True;
   }
   // Set LastInputState to CurrentState.
   LastInputState=CurrentInputState;
   return(ReturnVal);
}

/*************************************************/
/************  Check IR sensor *******************/
/*************************************************/

boolean CheckIREvents (void) 
{
   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentIR1State;
   static char LastIR1State;
   static char CurrentIR2State;
   static char LastIR2State;
   int analog;
 //  printf("Ch7 = ");
   // Local ReturnVal = False
   ReturnVal = False;
   CurrentIR1State = PTT & IR1;
   CurrentIR2State = PTT & IR2;

   if (CurrentIR1State != LastIR1State)
   {
      if(CurrentIR1State != 0)
      {
         puts("IR1 detected.\r\n");
         //PostEvent IRDetected with EventParam = 1 to DefaultSM
         ThisEvent.EventType = IRDetected;
         ThisEvent.EventParam = 0;
         PostDefaultSM(ThisEvent);
      }
   }

   if (CurrentIR2State != LastIR2State)
   {
      if(CurrentIR2State != 0)
      {
         puts("IR2 detected.\r\n");
         //PostEvent IRDetected with EventParam = 2 to DefaultSM
         ThisEvent.EventType = IRDetected;
         ThisEvent.EventParam = 1;
         PostDefaultSM(ThisEvent);
      }
   }
   //Reads the current state of the analog input, returns an event if it is in a certain range.
   analog = ADS12_ReadADPin(7);
   if(analog >= 100 && analog <= 150)
   {
      ThisEvent.EventType = AnalogCode;
      ThisEvent.EventParam = 1;
      printf("1");
      PostDefaultSM(ThisEvent);
         
   }
   
   if(analog >= 350 && analog <= 400)
   {
      ThisEvent.EventType = AnalogCode;
      ThisEvent.EventParam = 2;
      printf("2");
      PostDefaultSM(ThisEvent);
         
   }
   
   if(analog >= 600 && analog <= 650)
   {
      ThisEvent.EventType = AnalogCode;
      ThisEvent.EventParam = 3;
      printf("3");
      PostDefaultSM(ThisEvent);
         
   }
   if(analog >= 850 && analog <= 900)
   {
      ThisEvent.EventType = AnalogCode;
      ThisEvent.EventParam = 4;
      printf("4");
      PostDefaultSM(ThisEvent);
         
   }
   LastIR1State = CurrentIR1State;
   LastIR2State = CurrentIR2State;
   
   return(ReturnVal);
}

boolean CheckAnalogEvents (void) 
{
   // Declare variables.
   boolean ReturnVal;
   ES_Event currEvent;
   // Local ReturnVal = False
   ReturnVal = False;     
//   printf("Ch7 = %d.\r\n", ADS12_ReadADPin(7));
  

   
   
   return(ReturnVal);
}