/****************************************************************************
 Module
   MorseDecodeFSM.c

 Revision
   1.0.1

 Description
   MorseDecodeSM (implements the ?state machine? for Morse Decode, only 1 state)
   The EventType field of ThisEvent will be one of: DotDetectedEvent, DashDetectedEvent, EOCDetected, EOWDetected, ButtonDown.
   Returns True if No Error detected, False otherwise

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 11/5/12        ddg      Lab4
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "DecodeMorseFSM.h"
#include <stdio.h>
#include "ES_ServiceHeaders.h"

/*----------------------------- Module Defines ----------------------------*/

/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine
*/

static char DecodeMorse( void );
static void ClearMorseChar( void );
/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
//static DecodeMorseState_t CurrentState;
static char MorseString[9];
char legalChars[] = "ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890?.,:'-/()\"= !$&+;@_";
char morseCode[][9] ={ ".-","-...","-.-.","-..",".","..-.","--.",
                      "....","..",".---","-.-",".-..","--","-.","---",
                      ".--.","--.-",".-.","...","-","..-","...-",
                      ".--","-..-","-.--","--..",".----","..---",
                      "...--","....-",".....","-....","--...","---..",
                      "----.","-----","..--..",".-.-.-","--..--",
                      "---...",".----.","-....-","-..-.","-.--.-",
                      "-.--.-",".-..-.","-...-","-.-.--","...-..-",
                      ".-...",".-.-.","-.-.-.",".--.-.","..--.-"
                     };

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;
static int NextIndex;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitDecodeMorseFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away the priority, sets up the initial transition and does any 
     other required initialization for this state machine
 Notes

 Author
     Dustin D. Gerrard 11/5/12
****************************************************************************/
boolean InitDecodeMorse( uint8_t Priority )
{
 //Initialize the MyPriority variable with the passed in parameter.
  MyPriority = Priority;
  
  // Call ClearMorseChar.
  ClearMorseChar();
  
  // End.
  return True;
  
}

/****************************************************************************
 Function
     PostDecodeMorseService

 Parameters
     EF_Event ThisEvent , the event to post to the queue

 Returns
     boolean False if the Enqueue operation failed, True otherwise

 Description
     Posts an event to this state machine's queue
     
 Notes

 Author
      Dustin D. Gerrard 11/5/12
****************************************************************************/
boolean PostDecodeMorseService( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}

/****************************************************************************
 Function
    RunDecodeMorseService

 Parameters
   ES_Event : the event to process

 Returns
   ES_Event, ES_NO_EVENT if no error ES_ERROR otherwise

 Description
   add your description here
   
 Notes
   uses nested switch/case to implement the machine.
   
 Author
   Dustin D. Gerrard 11/5/12
****************************************************************************/
ES_Event RunDecodeMorseService( ES_Event ThisEvent )
{
   // Declare variables.
  ES_Event ReturnEvent;
  static char CurrChar;
  int SizeOfArray = 8;

  //CurrentState = DecodingMorse;  
  ReturnEvent.EventType = ES_NO_EVENT; // assume no errors
 
 // Based on the state of the ThisEvent variable choose one of the following blocks of code:. 
  switch ( ThisEvent.EventType )
  {
      case DotDetected :
         // If there is room for another Morse element in the internal representation. 
          if (NextIndex <= (SizeOfArray-1) )
          {
             // Add a Dot to the internal representation.
             MorseString[NextIndex]='.';
             
             // Increase Index by one
             NextIndex += 1;
                          
           } else
           {
             //return('Error');
           } break;

      case DashDetected :      
         // If there is room for another Morse element in the internal representation
          if (NextIndex <= (SizeOfArray-1) ) 
          {
             // Add a Dash to the internal representation.
             MorseString[NextIndex]='-';
             
             // Increase Index by one
             NextIndex += 1;

          } else
          {            
          }
          break;
          
      case EOC_Detected : 
          CurrChar = DecodeMorse();
          // If call to DecodeMorse does not return an error.
          if ( CurrChar != '#' )
          {            
            // Call ClearMorseChar.
            ClearMorseChar();                   
           }  else
           {          
           } break;
           
      case EOW_Detected : 
          CurrChar = DecodeMorse();
          // If call to DecodeMorse does not return an error.
          if ( CurrChar != '#' )
          {
             
             // Call ClearMorseChar.
             ClearMorseChar();                    
           } else
           {            
           } break;
                 
     
  }                                   // end switch on Current State
  return ReturnEvent;
}


/***************************************************************************
 private functions
 ***************************************************************************/

 /****************************************************************************
 Function ClearMorseChar

 Parameters
   ES_Event : the event to process

 Returns
   nothing

 Description
   Takes no parameters, returns either a character or a symbolic value indicating failure
   
 Notes
   uses nested switch/case to implement the machine.
   
 Author
   Dustin D. Gerrard 11/5/12
****************************************************************************/
static void ClearMorseChar( void )
{
   // Declare variables.
   int i;
   for(i = 0; i<8; i++)
   {
     // Clear (empty) the MorseString variable.
     MorseString[i] = '\0';
     NextIndex = 0;
   }
}

 /****************************************************************************
 Function DecodeMorse 
 
 Parameters
   ES_Event : the event to process

 Returns
   nothing

 Description
   Takes no parameters, returns either a character or a symbolic value indicating failure
   
 Notes
   uses nested switch/case to implement the machine.
   
 Author
   Dustin D. Gerrard 11/5/12
****************************************************************************/
static char DecodeMorse( void )
{
   //int jj;
   ES_Event currEvent;

//  printf("%s", MorseString);
   
//   printf("%d", strcmp(MorseString, "........"));
   if(strcmp(MorseString, "........") == 0)
   {
//      puts("@@@@@@@@@@@@@@@@@@@@@");
      currEvent.EventType = BATID_FOUND;
      currEvent.EventParam = 0;
      PostDefaultSM(currEvent);   
   }
   
   return 'a';
   /*
   // For every entry in the array MorseCode.
   for (jj=0; jj < 100; jj++)
   {
      // If MorseString is the same as current position in MorseCode .
      if(strcmp(MorseString, morseCode[jj]) == 0)
      { 
        // return contents of current position in LegalChars.
        return legalChars[jj];
      } 
   }
   */
   
}
