

/*---------------------------- Module Includes ----------------------------*/

#include <ME218_C32.h>
#include "ES_Configure.h"
#include "ES_General.h"
#include "ES_Events.h"
#include "ES_PostList.h"
#include "ES_Timers.h"
#include "ES_Framework.h"
#include "EventCheckers.h"
#include <stdio.h>
#include <stdlib.h>
#include "ES_ServiceHeaders.h"


/*----------------------------- Module Defines ----------------------------*/

#define QUEUE_SIZE 10
#define SERVOMIN 50
#define SERVOMAX 110

/*--------------------------- Module Prototypes ---------------------------*/

DefaultState processKeyboard(uint16_t);

/*---------------------------- Module Variables ---------------------------*/

static DefaultState CurrentState;
static uint8_t MyPriority;
static int angleServo;
static int basket;

static int timerActive;

/*---------------------------- Module Functions ---------------------------*/

// Initialization and Post functions taken from Ed Carryer's examples
boolean InitDefaultSM ( uint8_t Priority ) 
{ 
  ES_Event ThisEvent;
  
  MyPriority = Priority; 

  DDRM = 0x38;
  DDRT = 0x80;
  PTM = 0x0;
  
  Servo12_Init("Sxxxxxxx");
  printf("Servo Init.\r\n");
  angleServo = SERVOMIN;
  basket = rand() % 2;
  
  timerActive = 0;
  //Sends the correct basket number to Batman so Batman's machine knows too.
  if(basket)
  {
   sendMessage(3);
  } else 
  {
   sendMessage(4);
  }
  //Sets all AD pins to ouputs.
  if (ADS12_Init("OOOOOOOO") == ADS12_OK) 
  {
   
  } else 
  {
      printf("ADSInit Failed.\r\n");
  }
  
  CurrentState = InitDefault;
  puts("Initialized Default");
  ThisEvent.EventType = ES_INIT;
  
  if (ES_PostToService( MyPriority, ThisEvent) == True)
  { 
      puts("Returning True");
      return True; 
  } else { 
      return False; 
  } 
} 


boolean PostDefaultSM( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);

}

// Morse Code State Machine
ES_Event RunDefaultSM( ES_Event ThisEvent ) 
{ 
	DefaultState nextState = CurrentState;
	DefaultState keyState = NO_CHANGE;
	ES_Event ReturnEvent;
	ReturnEvent.EventType = ES_NO_EVENT;
	
	//timerActive keeps track of timer 7 for the lever speed.
	if (ThisEvent.EventType == ES_TIMEOUT && ThisEvent.EventParam == 7)
	{
	   timerActive = 0;        
	}
	//If Robin finds Batman failed, this resets the machine and puts it into lockdown.
	if (ThisEvent.EventType == Failure)
	{
	   AllOff();
	   PulseBuzzer(2000);
	   nextState = Lockdown;
	   ReturnEvent.EventType = FINISHED;
		PostLEDCodeSM(ReturnEvent);
	}
   switch (CurrentState)
	{
	   case InitDefault:
	      puts("In InitDefault");
	      if (ThisEvent.EventType == ES_INIT)
	      {
	         nextState = Lockdown;
	      }
	      break;
	      
	   case Lockdown:
	     
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }
		  if (ThisEvent.EventType == BATID_FOUND)
	      {
		//Resets everything and rerandomizes everything.
			 ResetButtons();
	         angleServo = SERVOMIN;
	         Servo(angleServo);
	         basket = rand() % 2;
	         printf("%d basket\r\n", basket);
	         if(basket)
            {
              sendMessage(3);
            } else 
            {
              sendMessage(4);
            }
	         nextState = WaitingForCode;
	         LightLED(4);
	         puts("BatID DETECTED.\n\r");
	      }
	      break;
	      

	   case WaitingForCode:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		 if (ThisEvent.EventType == CodeSuccess)
		  {
		     sendMessage(1);
		     OffLED(4);
			  puts("Changing state to WaitingForIR.\r\n");
			  nextState = WaitingForIR;
		  }
		  break;

	   case WaitingForIR:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == IRDetected)
		  {
			  puts("Changing state to WaitingForBall.\r\n");
			  printf("%d\r\n", basket);
			  if(basket)
			  {
			   LightLED(5);
			  } else 
			  {
			   LightLED(6);
			  }
			  nextState = WaitingForBall;
		  }
		  break;

	   case WaitingForBall:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }

		  if (ThisEvent.EventType == BallSuccess && ThisEvent.EventParam == basket)
		  //if(ThisEvent.EventType == ButtonDown)
		  {
		     //Correct basket
		     OffLED(5);
		     OffLED(6);
		     LightLED(3);
		     sendMessage(2);
			  puts("Changing state to WaitingForAnalog.\r\n");
			  nextState = WaitingForAnalog;
		  } else if (ThisEvent.EventType == BallSuccess && ThisEvent.EventParam != basket)
		  {
		    //Wrong basket
		    printf("RobinFailed.\r\n");
		    sendMessage(6);
		    AllOff();
		    PulseBuzzer(2000);
		    nextState = Lockdown;
		    ReturnEvent.EventType = FINISHED;
		    PostLEDCodeSM(ReturnEvent);
		  }
		  break;

	   case WaitingForAnalog:
		  if (ThisEvent.EventType == ES_NEW_KEY)
	      {
	         keyState = processKeyboard(ThisEvent.EventParam);
	         if (keyState != NO_CHANGE) nextState = keyState;
	      }
        if (ThisEvent.EventType == LeverDown && timerActive == 0)
        {
	    //Moves at most 1 degree every 70ms. Approximately 15 degrees per second.
            ES_Timer_SetTimer(7,70);
            ES_Timer_StartTimer(7);
            timerActive = 1;
            Servo(angleServo++);
            if(angleServo > SERVOMAX) angleServo == SERVOMAX;
            if(angleServo < SERVOMIN) angleServo == SERVOMIN;
        }
		  if (ThisEvent.EventType == AnalogCodeSuccess)
		  {
		     Celebration();
		     AllOff();
			  puts("Changing state to Victory!.\r\n");
			  nextState = Lockdown;
			  ReturnEvent.EventType = FINISHED;
	        PostLEDCodeSM(ReturnEvent);
	        Servo(SERVOMIN);
	        angleServo = SERVOMIN;
		  }
		  break;
	}
	CurrentState = nextState;
	ReturnEvent.EventType = ES_NO_EVENT;
	return ReturnEvent;
} 


DefaultState QueryDefaultSM ()
{
   return(CurrentState);
}

DefaultState processKeyboard(uint16_t input)
{
   ES_Event currEvent;
   char key = input;
   DefaultState newState = NO_CHANGE;
   if (key == 'a')
   {
	  currEvent.EventType = BATID_FOUND;
				PostDefaultSM(currEvent);
				PostLEDCodeSM(currEvent);

   }   
   if (key == 'b')
   {
      //Light LED
      newState = Lockdown;
	  currEvent.EventType = FINISHED;
	  currEvent.EventParam = 0;
	  PostLEDCodeSM(currEvent);
	  puts("In Lockdown now.\n\r");
	  
   }
   if (key == 'c')
   {
      currEvent.EventType = IRDetected;
				PostDefaultSM(currEvent);
   }
   if (key == '1')
   {
      sendMessage(1);
      LightLED(0);
	  puts("Sending Message 1.\n\r");	  	  
   }
   if (key == '2')
   {
      sendMessage(2);
      LightLED(1);
	  puts("Sending Message 2.\n\r");      
   }
   if (key == '3')
   {
      sendMessage(3);
      LightLED(2);
	  puts("Sending Message 3.\n\r");
   }
   if (key == '4')
   {
      sendMessage(4);
      LightLED(3);
	  puts("Sending Message 4.\n\r");      
   }
   if (key == '5')
   {
      sendMessage(5);
      LightLED(4);
	  puts("Sending Message 5.\n\r");      
   }
   if (key == '6')
   {
      sendMessage(6);
      LightLED(5);
	  puts("Sending Message 6.\n\r");      
   }
   if (key == '7')
   {
      sendMessage(7);
      LightLED(6);
	  puts("Sending Message 7.\n\r");      
   }
   if (key == '8')
   {
      sendMessage(8);
      LightLED(7);
	  puts("Buzzer on!.\n\r");
	  DDRT |= BIT6HI;
	  PTT |= BIT6HI;      
   }
   if (key == '9')
   {
      sendMessage(9);
      OffLED(0);
      OffLED(1);
      OffLED(2);
      OffLED(3);
      OffLED(4);
      OffLED(5);
      OffLED(6);
      OffLED(7);
      
	  puts("Buzzer off!.\n\r");
     DDRT |= BIT6HI;
	  PTT &= BIT6LO;
   }
  return newState;
}