 
/* ----------------------------------- Module Includes ----------------------------------- */

#include <ME218_C32.h>
#include "ES_Configure.h"
#include "ES_General.h"
#include "ES_Events.h"
#include "ES_PostList.h"
#include "ES_Timers.h"
#include "EventCheckers.h"
#include <stdio.h>
#include "ES_ServiceHeaders.h"

/*----------------------------- Module Defines ----------------------------*/

//These values should only be used with the M port on the C32.

#define INPUT_MASK 0x07 //0x00000111
#define MSG1 0x01 //0x00000001
#define MSG2 0x02 //0x00000010
#define MSG3 0x04 //0x00000100
#define MSG4 0x03 //0x00000011
#define MSG5 0x05 //0x00000101
#define MSG6 0x06 //0x00000110
#define MSG7 0x07 //0x00000111

//These values should only be used with the T port on the C32.
/* Button 1 on PTT Pin 0, Button 2 on PTT Pin 1, Button 3 on PTT Pin 2, Switch 1 on PTT Pin 3, Switch 2 on PTT Pin 4, Lever on PTT Pin 5*/
#define Button1		0x01 //0x00000001
#define Button2      0x02 //0x00000010
#define Button3		0x04 //0x00000100
#define Switch1		0x08 //0x00001000
#define Switch2		0x10 //0x00010000
#define Lever        0x20 //0x00100000

// PTT PIN 7 IS USED BY SERVO CONTROL
// PTT PIN 6 CURRENTLY UNUSED

/*--------------------------- Module Prototypes ---------------------------*/



/*---------------------------- Module Variables ---------------------------*/



/*---------------------------- Module Functions ---------------------------*/



// Checks for input messages from second C32. Bits in INPUT_MASK will be pulsed to send a message.
// checkMessage simply checks for this pulse.
boolean checkMessage (void)
{
	static unsigned char lastInputState;

	unsigned char currentInputState;
	boolean returnVal = FALSE;
	ES_Event currEvent;
	currEvent.EventParam = 0;

	currentInputState = PTM & INPUT_MASK;
	if (currentInputState != lastInputState)
	{
		switch (currentInputState)
		{
			//Message 1 is ___BatID Detected by Batman___
			case MSG1:
				puts("Received __BatID__! \r\n");

				currEvent.EventType = BATID_FOUND;
				PostDefaultSM(currEvent);
				PostLEDCodeSM(currEvent);

				break;

			//Message 2 is ___Button Pressed by Batman___
			case MSG2:
				puts("Received __Start Button__! \r\n");

		//		currEvent.EventType = StartButton;
		//		PostDefaultSM(currEvent);

				break;

			//Message 3 is ___IR Detected by Batman___
			case MSG3:
				puts("Received __IR Detected__! \r\n");

				currEvent.EventType = IRDetected;
				PostDefaultSM(currEvent);

				break;

			//Message 4 is ___Analog Code in by Batman___
			case MSG4:
				puts("Received __Analog Code__! \r\n");

				currEvent.EventType = AnalogCodeSuccess;
				PostDefaultSM(currEvent);

				break;
				
			case MSG5:
				puts("Received __Buzzer__! \r\n");

				currEvent.EventType = Buzzer;
				PostOutputsSM(currEvent);

				break;	
		
			case MSG6:
				puts("Received __Failure__! \r\n");

				currEvent.EventType = Failure;
				PostOutputsSM(currEvent);
				PostDefaultSM(currEvent);


				//currEvent.EventType = Msg1Event;
				//Post_____SM(currEvent);

				break;	
		
			//Message 7 is ___Batman Failed___
			case MSG7:
			//	puts("Received __Failure__! \r\n");

		//		currEvent.EventType = Failure;
	//			PostDefaultSM(currEvent);

				break;
				
			default:
			   break;

		}
	}
	lastInputState = currentInputState;
	return returnVal;

}


/*********************************************************/
/************  Robin: Code Button Checkers ***************/
/*********************************************************/

/*-------  Code Button 1  -------*/
boolean CheckButton1Events ( void ) 
{
   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentButton1State;
   static char LastButton1State;

   // Local ReturnVal = False
   ReturnVal = False;
   CurrentButton1State = PTT & Button1;
  

   // If CurrentState is READY2SAMPLE (debounce interval expired).
   if (QueryButton1SM () == READY2SAMPLE)
   {
	   // Post START_DEBOUNCE event to Button queue
	   ThisEvent.EventType = START_DEBOUNCE;
	   ThisEvent.EventParam = 0;
	   PostButtonFSM(ThisEvent);

	   // Set ReturnVal = True.   
	   ReturnVal = True;
   
	   // If the current button state is different from the LastButtonState
	   if (CurrentButton1State != LastButton1State)
	   {
   
	   // If the current button state is down.
		  if (CurrentButton1State != 0)
		  {
      
			 puts("\r\nButton 1 down.\n\r");
			 ThisEvent.EventType = ButtonDown; 
			 ThisEvent.EventParam = 1;
			 PostDefaultSM(ThisEvent);
			 PostLEDCodeSM(ThisEvent);
		   }
	   }  
   
	   LastButton1State = CurrentButton1State;
   
   } else {
   // Do nothing.
   }

   return(ReturnVal);
}       

/*-------  Code Button 2  -------*/
boolean CheckButton2Events ( void ) 
{
   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentButton2State;
   static char LastButton2State;

   // Local ReturnVal = False
   ReturnVal = False;
   CurrentButton2State = PTT & Button2;

   // If CurrentState is READY2SAMPLE (debounce interval expired).
   if (QueryButton2SM () == READY2SAMPLE)
   {
	   // Post START_DEBOUNCE event to Button queue
	   ThisEvent.EventType = START_DEBOUNCE;
	   ThisEvent.EventParam = 1;
	   PostButtonFSM(ThisEvent);

	   // Set ReturnVal = True.   
	   ReturnVal = True;
   
	   // If the current button state is different from the LastButtonState
	   if (CurrentButton2State != LastButton2State)
	   {
	   // If the current button state is down.
		  if (CurrentButton2State != 0)
		  {
      
			  puts("\r\nButton 2 down.\n\r");
			  ThisEvent.EventType = ButtonDown; 
			  ThisEvent.EventParam = 2;
			  PostDefaultSM(ThisEvent);
			  PostLEDCodeSM(ThisEvent);
		  }
	   }  
   
	   LastButton2State = CurrentButton2State;
   
   } else {
   // Do nothing.
   }

   return(ReturnVal);
} 

/*-------  Code Button 3  -------*/
boolean CheckButton3Events ( void ) 
{

   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentButton3State;
   static char LastButton3State;

   // Local ReturnVal = False
   ReturnVal = False;
   CurrentButton3State = PTT & Button3;

   // If CurrentState is READY2SAMPLE (debounce interval expired).
   if (QueryButton3SM () == READY2SAMPLE)
   {
	   // Post START_DEBOUNCE event to Button queue
	   ThisEvent.EventType = START_DEBOUNCE;
	   ThisEvent.EventParam = 2;
	   PostButtonFSM(ThisEvent);

	   // Set ReturnVal = True.   
	   ReturnVal = True;
   
	   // If the current button state is different from the LastButtonState
	   if (CurrentButton3State != LastButton3State)
	   {
	   // If the current button state is down.
		  if (CurrentButton3State != 0)
		  {
      
		     puts("Button 3 down.\n\r");
			 ThisEvent.EventType = ButtonDown; 
			 ThisEvent.EventParam = 3;
			 PostDefaultSM(ThisEvent);
			 PostLEDCodeSM(ThisEvent);
		   }
	   }  
   
	   LastButton3State = CurrentButton3State;
   
   } else {
   // Do nothing.
   }

   return(ReturnVal);
} 

/*************************************************/
/***********  Check basket switches **************/
/*************************************************/

//------ Switch 1 -------

boolean CheckSwitch1Events ( void ) 
{
   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentSwitch1State;
   static char LastSwitch1State;

   // Local ReturnVal = False
   ReturnVal = False;
   CurrentSwitch1State = PTT & Switch1;

   // If CurrentState is READY2SAMPLE (debounce interval expired).
   if (QuerySwitch1SM () == READY2SAMPLE)
   {
	   // Post START_DEBOUNCE event to Button queue
	   ThisEvent.EventType = START_DEBOUNCE;
	   ThisEvent.EventParam = 3;
	   PostButtonFSM(ThisEvent);

	   // Set ReturnVal = True.   
	   ReturnVal = True;
   
	   // If the current button state is different from the LastButtonState
	   if (CurrentSwitch1State != LastSwitch1State)
	   {
	   // If the current button state is down.
		  if (CurrentSwitch1State != 0)
		  {
			 puts("Switch 1 down.\n\r");
			 ThisEvent.EventType = BallSuccess; 
			 ThisEvent.EventParam = 0;
			 PostDefaultSM(ThisEvent);
		   }
	   }  
   
	   LastSwitch1State = CurrentSwitch1State;
   
   } else {
   // Do nothing.
   }

   return(ReturnVal);
}       

// ------ Switch 2 -------
boolean CheckSwitch2Events ( void ) 
{
   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentSwitch2State;
   static char LastSwitch2State;

   // Local ReturnVal = False
   ReturnVal = False;
   CurrentSwitch2State = PTT & Switch2;


   // If CurrentState is READY2SAMPLE (debounce interval expired).
   if (QuerySwitch2SM () == READY2SAMPLE)
   {
	   // Post START_DEBOUNCE event to Button queue
	   ThisEvent.EventType = START_DEBOUNCE;
	   ThisEvent.EventParam = 4;
	   PostButtonFSM(ThisEvent);

	   // Set ReturnVal = True.   
	   ReturnVal = True;
	   
	   // If the current button state is different from the LastButtonState
	   if (CurrentSwitch2State != LastSwitch2State)
	   {
	   // If the current button state is down.
		  if (CurrentSwitch2State != 0)
		  {
			 puts("Switch 2 down.\n\r");
			 ThisEvent.EventType = BallSuccess; 
			 ThisEvent.EventParam = 1;
			 PostDefaultSM(ThisEvent);
		   }
	   }  
   
	   LastSwitch2State = CurrentSwitch2State;
   
   } else {
   // Do nothing.
   }

   return(ReturnVal);
}  

/*************************************************/
/****************  Check Lever *******************/
/*************************************************/

boolean CheckLeverEvents ( void ) 
{

   // Declare variables.
   ES_Event ThisEvent;
   boolean ReturnVal;
   static char CurrentLeverState;
   static char LastLeverState;

   // Local ReturnVal = False
   ReturnVal = False;
   CurrentLeverState = PTT & Lever;

if(CurrentLeverState)
{
   ThisEvent.EventType = LeverDown; 
			 ThisEvent.EventParam = 0;
			 PostDefaultSM(ThisEvent);
}
   // If CurrentState is READY2SAMPLE (debounce interval expired).
   if (QueryLeverSM () == READY2SAMPLE)
   {
	   // Post START_DEBOUNCE event to Button queue
	   ThisEvent.EventType = START_DEBOUNCE;
	   ThisEvent.EventParam = 5;
	   PostButtonFSM(ThisEvent);

	   // Set ReturnVal = True.   
	   ReturnVal = True;
   
	   // If the current button state is different from the LastButtonState
	   if (CurrentLeverState != LastLeverState)
	   {
	   // If the current button state is down.
		  if (CurrentLeverState != 0)
		  {
      			//Want to post constant LeverDown events so that the servo moves slowly.
			 puts("Lever down.\n\r");
			 // PostEvent ButtonDown to MorseElements & DecodeMorse queues
			 ThisEvent.EventType = LeverDown; 
			 ThisEvent.EventParam = 0;
			 PostDefaultSM(ThisEvent);
		  } else {
			 puts("Lever up. \n\r");
			 ThisEvent.EventType = LeverUp; 
			 ThisEvent.EventParam = 0;
			 PostDefaultSM(ThisEvent);
		  }

	   }  
   
	   LastLeverState = CurrentLeverState;
   
   } else {
   // Do nothing.
   }

   return(ReturnVal);
} 