/****************************************************************************
 Module
   MorseElementsFSM.c

 Revision
   1.0.1

 Description
   This is a template file for implementing flat state machines under the 
   Gen2 Events and Services Framework.

 Notes

 History
 When           Who     What/Why
 -------------- ---     --------
 11/04/12 23:55 ddg      create Button.c from TemplateFSM.c
 01/15/12 11:12 jec      revisions for Gen2 framework
 11/07/11 11:26 jec      made the queue static
 10/30/11 17:59 jec      fixed references to CurrentEvent in RunTemplateSM()
 10/23/11 18:20 jec      began conversion from SMTemplate.c (02/20/07 rev)
****************************************************************************/
/*----------------------------- Include Files -----------------------------*/
/* include header files for this state machine as well as any machines at the
   next lower level in the hierarchy that are sub-machines to this machine
*/
#include "ES_Configure.h"
#include "ES_Framework.h"
#include "MorseElementsFSM.h"
#include "ME218_C32.h"
#include "ES_ServiceHeaders.h"
#include <stdio.h>
#include <stdlib.h>

/*----------------------------- Module Defines ----------------------------*/


/*---------------------------- Module Functions ---------------------------*/
/* prototypes for private functions for this machine.They should be functions
   relevant to the behavior of this state machine*/
   
void getNewRand( void );

/*---------------------------- Module Variables ---------------------------*/
// everybody needs a state variable, you may need others as well.
// type of state variable should match htat of enum in header file
static LEDState CurrentState;
static int n1;
static int n2;
static int n3;
static int n4;

// with the introduction of Gen2, we need a module level Priority var as well
static uint8_t MyPriority;


/*------------------------------ Module Code ------------------------------*/
/****************************************************************************
 Function
     InitializeMorseElementsFSM

 Parameters
     uint8_t : the priorty of this service

 Returns
     boolean, False if error in initialization, True otherwise

 Description
     Saves away the priority, sets up the initial transition and does any 
     other required initialization for this state machine
 Notes

 Author
     Dustin D. Gerrard 11/5/12
****************************************************************************/
boolean InitializeLEDCode ( uint8_t Priority )
{
   //Declare variables.
   ES_Event NewEvent;
  
   // Initialize the MyPriority variable with the passed in parameter.
   MyPriority = Priority;

  
   // Set CurrentState to be InitMorseElements
   CurrentState = InitLEDCode;
   NewEvent.EventType = ES_INIT;
   PostLEDCodeSM(NewEvent);
   
 /*  int i;
   for (i = 0; i < 100; i++)
   {
      rand();
   }*/
  
   
   n1 = rand() % 3 + 1;
   n2 = rand() % 3 + 1;
   n3 = rand() % 3 + 1;
   n4 = rand() % 3 + 1;
   
   printf("%d, %d, %d, %d\r\n", n1, n2, n3, n4);
  
   // End. 
   return True;
}

boolean PostLEDCodeSM( ES_Event ThisEvent )
{
  return ES_PostToService( MyPriority, ThisEvent);
}


ES_Event LEDCodeSM( ES_Event ThisEvent )
{  
	// Declare variables.
	// Local Variables: NextState
	ES_Event currEvent;
	LEDState NextState; 
	currEvent.EventType = ES_NO_EVENT;

	// Set NextState to CurrentState.
	NextState = CurrentState;

   if(ThisEvent.EventType == FINISHED)
   {
     NextState = WaitToStart;
   }

	// Based on the state of the CurrentState variable choose one of the following blocks of code:
	switch ( CurrentState )
	{
	
	   
		   
	   case InitLEDCode:
		  if(ThisEvent.EventType == ES_INIT)
		  {
		  // Set NextState to CalWaitForRise
			 NextState = WaitToStart;         
		  }   
		  break;
		  
      case WaitToStart:
		  if(ThisEvent.EventType == BATID_FOUND)
		  {
		  // Set NextState to CalWaitForRise
		    getNewRand();
		    LEDSequence(n1, n2, n3, n4);
			 NextState = WaitForFirstButton;         
		  }   
		  break;

	   case WaitForFirstButton:
	     
		  
		  if(ThisEvent.EventType == ButtonDown)
		  {
			  if(ThisEvent.EventParam == n1) 
			  {
				//If correct, proceed to next state.
			      NextState = WaitForSecondButton;
			      printf("First Correct\r\n");
			  } else {
				//If wrong, play the buzzer and put it back to the start of the code. 
			     PulseBuzzer(500);
				  NextState = WaitForFirstButton;	
			  }
		  }   
		  break;

	   case WaitForSecondButton:
		  if(ThisEvent.EventType == ButtonDown)
		  {
			  if(ThisEvent.EventParam == n2) 
			  {
			      NextState = WaitForThirdButton;
			      printf("Second Correct\r\n");
			  } else {
			     PulseBuzzer(500); 
				  NextState = WaitForFirstButton;	
			  }
		  }   
		  break;

	   case WaitForThirdButton:
		  if(ThisEvent.EventType == ButtonDown)
		  {
			  if(ThisEvent.EventParam == n3) 
			  {
			      NextState = WaitForFourthButton;
			      printf("Third Correct\r\n");
			  } else {
			     PulseBuzzer(500);
				  NextState = WaitForFirstButton;	
			  }
		  }   
		  break;

	   case WaitForFourthButton:
		  if(ThisEvent.EventType == ButtonDown)
		  {
			  if(ThisEvent.EventParam == n4) 
			  {
			     printf("Fourth Correct\r\n");
				  currEvent.EventType = CodeSuccess;
				  currEvent.EventParam = 0;
				  PostDefaultSM(currEvent);
			     NextState = Idle;
			  } else {
		   	  PulseBuzzer(500);
				  NextState = WaitForFirstButton;	
			  }
		  }   
		  break;

	   case Idle:
         if(ThisEvent.EventType == FINISHED)
         {
            NextState = WaitToStart;
         }
		   break;

	   default:
		   break;
	}
  
   CurrentState = NextState;  
   currEvent.EventType = ES_NO_EVENT;           
   return currEvent;
} 


 LEDState QueryLEDCodeSM ( void )
{
   return(CurrentState);
}

//getNewRand rerandomizes the 4 numbers to display. 
void getNewRand( void)
{
   n1 = rand() % 3 + 1;
   n2 = rand() % 3 + 1;
   n3 = rand() % 3 + 1;
   n4 = rand() % 3 + 1;
   
   printf("%d, %d, %d, %d\r\n", n1, n2, n3, n4);
}
