

/*---------------------------- Module Includes ----------------------------*/

#include <ME218_C32.h>
#include "ES_Configure.h"
#include "ES_General.h"
#include "ES_Events.h"
#include "ES_PostList.h"
#include "ES_Timers.h"
#include "ES_Framework.h"
#include "EventCheckers.h"
#include <stdio.h>
#include "ES_ServiceHeaders.h"


/*----------------------------- Module Defines ----------------------------*/

//These values should only be used with the T port on the C32.
//Servo control on T7

//These values should only be used with the AD port on the C32.
#define LED0 BIT0_HI       //CodeLight1
#define LED1 BIT1_HI       //CodeLight2
#define LED2 BIT2_HI       //CodeLight3
#define LED3 BIT3_HI       //LeverLight
#define LED4 BIT4_HI       //CodeButtonLight
#define LED5 BIT5_HI       //Basket1Light
#define LED6 BIT6_HI       //Basket2Light
#define LED7 BIT7_HI       //Buzzer

#define SERVOMIN 50
#define SERVOMAX 110

/*--------------------------- Module Prototypes ---------------------------*/

static void Wait ( int );

/*---------------------------- Module Variables ---------------------------*/

static uint8_t MyPriority;

//static boolean LeverStatus;


/*---------------------------- Module Functions ---------------------------*/

boolean InitOutputs ( uint8_t Priority )  
{
   ES_Event ThisEvent; 
   MyPriority = Priority; 
   ThisEvent.EventType = ES_INIT; 
   ES_Timer_SetTimer(6,500);
   ES_Timer_StartTimer(6);
   Servo(SERVOMIN);
   if (ES_PostToService( MyPriority, ThisEvent) == True)
   { 
       puts("Returning True");
       return True; 
   } else { 
       return False; 
   } 
   
}

boolean PostOutputsSM( ES_Event ThisEvent )
{
	return ES_PostToService( MyPriority, ThisEvent);

}

ES_Event RunOutputsSM( ES_Event ThisEvent )
{
   ES_Event ReturnEvent;
	ReturnEvent.EventType = ES_NO_EVENT;
	//If OutputsSM gets the Failure, pulses the buzzer for 2 seconds to indicate failure.
	if (ThisEvent.EventType == Failure)
	{
	   PulseBuzzer(2000);
	}
	//Turns off the buzzer when timer 0 times out.
	if (ThisEvent.EventType == ES_TIMEOUT && ThisEvent.EventParam == 6)
	{
	   OffLED(7);
	}
	return ReturnEvent;
}

//Preferably only used with buzzer (7)
void PulseBuzzer(int i)
{
   LightLED(7);
   ES_Timer_SetTimer(6,i);
   ES_Timer_StartTimer(6);
}

void LightLED(int i)
{
printf("Lighting LED %d.\r\n", i);
   //Shifts bits by i and turns the bit high.
   PTAD |= (0x1 << i);   
}

void OffLED(int i)
{
printf("Off LED %d.\r\n", i);
   //Shifts bits by i and turns the bit low.
   PTAD &= ~(0x1 << i);
}

void OffAll(void )
{
   //Turns off all the outputs and doesn't touch the analog input.
   PTAD &= (0x1 << 7);
}

void AllOn(void)
{
   PTAD = ~(0x00);
}

//Celebration uses blocking code because when it happens it shouldn't take any inputs.
void Celebration(void )
{
   int i;
   for(i = 0; i < 5; i++)
   {
      AllOn();
      Wait(700);
      AllOff();
      Wait(300);
   }
}

//This Sequence is blocking code because there shouldn't be any inputs taken in.
void LEDSequence(int n1, int n2, int n3, int n4)
{
   LightLED(n1 - 1);
   Wait(750);
   OffLED(n1 - 1);
   Wait(250); 
   LightLED(n2 - 1);
   Wait(750);
   OffLED(n2 - 1);
   Wait(250); 
   LightLED(n3 - 1);
   Wait(750);
   OffLED(n3 - 1);
   Wait(250); 
   LightLED(n4 - 1);
   Wait(750);
   OffLED(n4 - 1);
   Wait(250);   
}

//This wait is blocking code
static void Wait(int ms)
{
   uint16_t time1 = ES_Timer_GetTime();
   while(ES_Timer_GetTime() <= time1 + ms)
   {
      
   }
}

//Turns servo to the angle given
void Servo(int angle)
{
   //Makes sure the servo doesn't go past the limits
   if(angle < SERVOMIN) angle = SERVOMIN;
   if(angle > SERVOMAX) angle = SERVOMAX;
   Servo12_SetPulseWidth(7, 600+10*angle);
}